@@ -43,7 +43,9 @@
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
-#include <termios.h> /* struct winsize */
+#if HAVE_TERMIOS
+# include <termios.h> /* struct winsize */
+#endif
 #include <unistd.h>
 
 #include "lowdown.h"
@@ -133,9 +135,19 @@ sandbox_pre(void)
 	/* Do nothing. */
 }
 
-#else /* No sandbox. */
+#else /* No compile-time sandbox. */
 
-#warning Compiling without sandbox support.
+/*
+ * WebAssembly (WASI) is a virtual architecture, so it's "sandboxed" by
+ * default.  Since building WASI targets is now fairly common, we can
+ * specifically address this by not issuing the warning.  XXX: this
+ * special-cases this output, which is not something that I like, but
+ * until there are other platforms that are similarly
+ * sandboxed-by-default, will keep as-is.
+ */
+#ifndef __wasi__
+# warning Compiling without sandbox support.
+#endif
 
 static void
 sandbox_post(int fdin, int fddin, int fdout)
@@ -156,12 +168,15 @@ sandbox_pre(void)
 static size_t
 get_columns(void)
 {
+	size_t res = 72;
+#if HAVE_TERMIOS
 	struct winsize	 size;
 
 	memset(&size, 0, sizeof(struct winsize));
-	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &size) == -1)
-		return 72;
-	return size.ws_col;
+	if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &size) != -1)
+		res = size.ws_col;
+#endif
+	return res;
 }
 
 /*
@@ -212,13 +227,14 @@ main(int argc, char *argv[])
 				*din = NULL;
 	const char		*fnin = "<stdin>", *fnout = NULL,
 	      	 		*fndin = NULL, *extract = NULL, *er,
-				*mainopts = "M:m:sT:t:o:X:",
+				*mainopts = "LM:m:sT:t:o:X:",
 	      			*diffopts = "M:m:sT:t:o:", *odtstyfn = NULL;
 	struct lowdown_opts 	 opts;
 	struct stat		 st;
 	int			 c, diff = 0, fd,
 				 status = 0, aoflag = 0, roflag = 0,
-				 aiflag = 0, riflag = 0, centre = 0;
+				 aiflag = 0, riflag = 0, centre = 0,
+				 list = 0;
 	char			*ret = NULL, *cp, *odtsty = NULL;
 	size_t		 	 i, retsz = 0, rcols, sz;
 	ssize_t			 ssz;
@@ -374,6 +390,7 @@ main(int argc, char *argv[])
 			opts.oflags |= LOWDOWN_STANDALONE;
 			break;
 		case 't':
+			/* FALLTHROUGH */
 		case 'T':
 			if (strcasecmp(optarg, "ms") == 0)
 				opts.type = LOWDOWN_NROFF;
@@ -398,6 +415,11 @@ main(int argc, char *argv[])
 			break;
 		case 'X':
 			extract = optarg;
+			list = 0;
+			break;
+		case 'L':
+			list = 1;
+			extract = NULL;
 			break;
 		case 0:
 			if (roflag)
@@ -537,7 +559,7 @@ main(int argc, char *argv[])
 
 	/* Require metadata when extracting. */
 
-	if (extract)
+	if (extract || list)
 		opts.feat |= LOWDOWN_METADATA;
 
 	/* 
@@ -570,6 +592,10 @@ main(int argc, char *argv[])
 			status = 1;
 			warnx("%s: unknown keyword", extract);
 		}
+	} else if (list) {
+		assert(!diff);
+		TAILQ_FOREACH(m, &mq, entries)
+			fprintf(fout, "%s\n", m->key);
 	} else
 		fwrite(ret, 1, retsz, fout);
 
@@ -596,11 +622,15 @@ main(int argc, char *argv[])
 usage:
 	if (!diff) {
 		fprintf(stderr, 
-			"usage: lowdown [-s] [input_options] [output_options] [-M metadata]\n"
-			"               [-m metadata] [-o output] [-t mode] [-X keyword] [file]\n");
+			"usage: lowdown [-Ls] [input_options] "
+			"[output_options] [-M metadata]\n"
+			"               [-m metadata] "
+			"[-o output] [-t mode] [-X keyword] [file]\n");
 	} else
 		fprintf(stderr, 
-			"usage: lowdown-diff [-s] [input_options] [output_options] [-M metadata]\n"
-			"                    [-m metadata] [-o output] [-t mode] oldfile [newfile]\n");
+			"usage: lowdown-diff [-s] [input_options] "
+			"[output_options] [-M metadata]\n"
+			"                    [-m metadata] "
+			"[-o output] [-t mode] oldfile [newfile]\n");
 	return 1;
 }
