@@ -49,6 +49,7 @@ struct 	nroff {
 	enum nfont		  fonts[NFONT__MAX]; /* see bqueue_font() */
 	struct bnodeq		**foots; /* footnotes */
 	size_t			  footsz; /* footnote size */
+	size_t			  indent; /* indentation width */
 };
 
 enum	bscope {
@@ -86,7 +87,7 @@ struct	bnode {
 TAILQ_HEAD(bnodeq, bnode);
 
 /*
- * Escape unsafe text into roff output such that no roff fetaures are
+ * Escape unsafe text into roff output such that no roff features are
  * invoked by the text (macros, escapes, etc.).
  * If "oneline" is non-zero, newlines are replaced with spaces.
  * If "literal", doesn't strip leading space.
@@ -735,36 +736,29 @@ rndr_blockcode(const struct nroff *st, struct bnodeq *obq,
 }
 
 static int
-rndr_definition_title(struct bnodeq *obq, struct bnodeq *bq)
+rndr_definition_title(const struct nroff *st, struct bnodeq *obq,
+     struct bnodeq *bq)
 {
+	char	 buf[32];
 
-	if (bqueue_block(obq, ".LP") == NULL)
+	snprintf(buf, sizeof(buf), ".TP %zu", st->indent);
+
+	if (st->man && bqueue_block(obq, buf) == NULL)
+		return 0;
+	else if (!st->man && bqueue_block(obq, ".XP") == NULL)
 		return 0;
 	TAILQ_CONCAT(obq, bq, entries);
+	if (st->man && bqueue_span(obq, "\n") == NULL)
+		return 0;
+	else if (!st->man && bqueue_block(obq, ".br") == NULL)
+		return 0;
 	return 1;
 }
 
 static int
-rndr_definition_data(struct bnodeq *obq, struct bnodeq *bq)
+rndr_definition_data(const struct nroff *st, struct bnodeq *obq,
+    struct bnodeq *bq)
 {
-	/*
-	 * The IP creates an empty vertical space til I figure out a
-	 * better way to do hanging lists, so account for it by backing
-	 * up first.
-	 *
-	 * XXX: this produces different results on mandoc and groff as
-	 * of 2022-02-19: mandoc backs up one space, while groff backs
-	 * up two.  The groff behaviour is what we want, so that the
-	 * text is flush on the next line, but this is good enough.
-	 */
-
-	if (bqueue_block(obq, ".if n \\\n.sp -1v") == NULL)
-		return 0;
-	if (bqueue_block(obq, ".if t \\\n.sp -0.25v\n") == NULL)
-		return 0;
-	if (bqueue_block(obq, ".IP \"\" \\*(PI") == NULL)
-		return 0;
-
 	/* Strip out leading paragraphs. */
 
 	bqueue_strip_paras(bq);
@@ -804,7 +798,6 @@ rndr_blockquote(struct nroff *st,
 	if (bqueue_block(obq, ".RS") == NULL)
 		return 0;
 	TAILQ_CONCAT(obq, bq, entries);
-	st->post_para = 1;
 	return bqueue_block(obq, ".RE") != NULL;
 }
 
@@ -949,17 +942,44 @@ rndr_link(struct nroff *st, struct bnodeq *obq, struct bnodeq *bq,
 }
 
 static int
-rndr_listitem(struct bnodeq *obq, const struct lowdown_node *n,
-	struct bnodeq *bq, const struct rndr_listitem *param)
+rndr_listitem(const struct nroff *st, struct bnodeq *obq,
+	const struct lowdown_node *n, struct bnodeq *bq,
+	const struct rndr_listitem *param)
 {
 	struct bnode	*bn;
 	const char	*box;
+	size_t		 total = 0, numsize;
+
+	/*
+	 * When indenting for the number of bullet preceding the line of
+	 * text, use "indent" spaces as the default.  For ordered lists
+	 * stretching beyond the indent size, enlarge as necessary.
+	 */
 
 	if (param->flags & HLIST_FL_ORDERED) {
+		if (n->parent != NULL &&
+		    n->parent->type == LOWDOWN_LIST)
+			total = n->parent->rndr_list.items +
+				(n->parent->rndr_list.start - 1);
+
+		/* Yes, a little ridiculous, but doesn't hurt. */
+
+		numsize = total < 10 ? 3 :
+			total < 100 ? 4 :
+			total < 1000 ? 5 :
+			total < 10000 ? 6 :
+			total < 100000 ? 7 :
+			total < 1000000 ? 8 :
+			total < 10000000 ? 9 :
+			10;
+
+		if (numsize < st->indent)
+			numsize = st->indent;
+
 		if ((bn = bqueue_block(obq, ".IP")) == NULL)
 			return 0;
 		if (asprintf(&bn->nargs, 
-		    "\"%zu.  \"", param->num) == -1)
+		    "\"%zu.\" %zu", param->num, numsize) == -1)
 			return 0;
 	} else if (param->flags & HLIST_FL_UNORDERED) {
 		if (param->flags & HLIST_FL_CHECKED)
@@ -970,7 +990,8 @@ rndr_listitem(struct bnodeq *obq, const struct lowdown_node *n,
 			box = "(bu";
 		if ((bn = bqueue_block(obq, ".IP")) == NULL)
 			return 0;
-		if (asprintf(&bn->nargs, "\"\\%s\" 2", box) == -1)
+		if (asprintf(&bn->nargs, "\"\\%s\" %zu",
+		    box, st->indent) == -1)
 			return 0;
 	}
 
@@ -988,12 +1009,10 @@ rndr_listitem(struct bnodeq *obq, const struct lowdown_node *n,
 	    (n->rndr_listitem.flags & HLIST_FL_DEF))
 		return 1;
 
-	if (TAILQ_NEXT(n, entries) != NULL) {
-		if (bqueue_block(obq, ".if n \\\n.sp -1") == NULL)
-			return 0;
-		if (bqueue_block(obq, ".if t \\\n.sp -0.25v\n") == NULL)
-			return 0;
-	}
+	if (TAILQ_NEXT(n, entries) != NULL &&
+	    (bqueue_block(obq, ".if n \\\n.sp -1") == NULL ||
+	     bqueue_block(obq, ".if t \\\n.sp -0.25v\n") == NULL))
+		return 0;
 
 	return 1;
 }
@@ -1160,7 +1179,6 @@ rndr_table(struct nroff *st, struct bnodeq *obq, struct bnodeq *bq)
 	if (bqueue_block(obq, "tab(|) expand allbox;") == NULL)
 		return 0;
 	TAILQ_CONCAT(obq, bq, entries);
-	st->post_para = 1;
 	return bqueue_block(obq, ".TE") != NULL;
 }
 
@@ -1691,7 +1709,7 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 	const struct lowdown_node *n, struct bnodeq *obq)
 {
 	const struct lowdown_node	*child;
-	int				 rc = 1;
+	int				 rc = 1, post_para = st->post_para;
 	enum nfont			 fonts[NFONT__MAX];
 	struct bnodeq			 tmpbq;
 	struct bnode			*bn;
@@ -1753,10 +1771,10 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		rc = rndr_list(st, obq, n, &tmpbq);
 		break;
 	case LOWDOWN_DEFINITION_DATA:
-		rc = rndr_definition_data(obq, &tmpbq);
+		rc = rndr_definition_data(st, obq, &tmpbq);
 		break;
 	case LOWDOWN_DEFINITION_TITLE:
-		rc = rndr_definition_title(obq, &tmpbq);
+		rc = rndr_definition_title(st, obq, &tmpbq);
 		break;
 	case LOWDOWN_DOC_HEADER:
 		rc = rndr_doc_header(st, obq, mq);
@@ -1775,7 +1793,8 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		rc = rndr_list(st, obq, n, &tmpbq);
 		break;
 	case LOWDOWN_LISTITEM:
-		rc = rndr_listitem(obq, n, &tmpbq, &n->rndr_listitem);
+		rc = rndr_listitem(st,
+			obq, n, &tmpbq, &n->rndr_listitem);
 		break;
 	case LOWDOWN_PARAGRAPH:
 		rc = rndr_paragraph(st, n, obq, &tmpbq);
@@ -1820,6 +1839,13 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		break;
 	case LOWDOWN_FOOTNOTE:
 		rc = rndr_footnote_ref(st, obq, &tmpbq);
+		/*
+		 * Restore what subsequent paragraphs should do.  This
+		 * macro will create output that's delayed in being
+		 * shown.  It might set post_para, which we don't want
+		 * to propogate to the actual output that will follow.
+		 */
+		st->post_para = post_para;
 		break;
 	case LOWDOWN_RAW_HTML:
 		rc = rndr_raw_html(st, obq, &n->rndr_raw_html);
@@ -1927,6 +1953,14 @@ lowdown_nroff_new(const struct lowdown_opts *opts)
 
 	p->flags = opts != NULL ? opts->oflags : 0;
 	p->man = opts != NULL && opts->type == LOWDOWN_MAN;
+
+	/*
+	 * Set a default indentation.  For -man, we use 3 because we
+	 * want to be efficient with page width.  For -ms, use 5 because
+	 * that'll usually be used for PDF with a variable-width font.
+	 */
+
+	p->indent = p->man ? 3 : 5;
 	return p;
 }
 
